@@ -3,9 +3,12 @@
 
 #include <cstddef> // size_t
 #include <dbarts/cstdint.hpp> // int types
+#include <limits> // numeric_limits
 
 #include <dbarts/random.hpp>
 
+#define DBARTS_CONTROL_INVALID_SEED static_cast<std::uint_least32_t>(std::numeric_limits<int>::min())
+
 namespace dbarts {
   struct BARTFit;
   
@@ -32,6 +35,7 @@ namespace dbarts {
     
     rng_algorithm_t rng_algorithm;
     rng_standardNormal_t rng_standardNormal;
+    std::uint_least32_t rng_seed;
     
     CallbackFunction callback;
     void* callbackData;
@@ -40,7 +44,8 @@ namespace dbarts {
       responseIsBinary(false), verbose(true), keepTrainingFits(true), useQuantiles(false), keepTrees(false),
       defaultNumSamples(800), defaultNumBurnIn(200), numTrees(75), numChains(1), numThreads(1), treeThinningRate(1),
       printEvery(100), printCutoffs(0), rng_algorithm(RNG_ALGORITHM_MERSENNE_TWISTER),
-      rng_standardNormal(RNG_STANDARD_NORMAL_INVERSION), callback(NULL), callbackData(NULL)
+      rng_standardNormal(RNG_STANDARD_NORMAL_INVERSION), rng_seed(DBARTS_CONTROL_INVALID_SEED),
+      callback(NULL), callbackData(NULL)
     { }
     Control(std::size_t defaultNumSamples,
             std::size_t defaultNumBurnIn,
@@ -50,20 +55,22 @@ namespace dbarts {
             std::uint32_t treeThinningRate,
             bool keepTrainingFits,
             bool verbose,
-            uint32_t printEvery,
+            std::uint32_t printEvery,
             bool responseIsBinary,
             bool useQuantiles,
             bool keepTrees,
-            uint32_t printCutoffs,
+            std::uint32_t printCutoffs,
             rng_algorithm_t rng_algorithm,
             rng_standardNormal_t rng_standardNormal,
+            std::uint_least32_t rng_seed,
             CallbackFunction callback,
             void* callbackData) :
-      responseIsBinary(responseIsBinary), verbose(verbose), keepTrainingFits(keepTrainingFits), useQuantiles(useQuantiles),
-      keepTrees(keepTrees), defaultNumSamples(defaultNumSamples), defaultNumBurnIn(defaultNumBurnIn), numTrees(numTrees),
+      responseIsBinary(responseIsBinary), verbose(verbose), keepTrainingFits(keepTrainingFits),
+      useQuantiles(useQuantiles), keepTrees(keepTrees),
+      defaultNumSamples(defaultNumSamples), defaultNumBurnIn(defaultNumBurnIn), numTrees(numTrees),
       numChains(numChains), numThreads(numThreads), treeThinningRate(treeThinningRate), printEvery(printEvery),
       printCutoffs(printCutoffs), rng_algorithm(rng_algorithm), rng_standardNormal(rng_standardNormal),
-      callback(callback), callbackData(callbackData)
+      rng_seed(rng_seed), callback(callback), callbackData(callbackData)
     { }
   };
 } // namespace dbarts
